#!/usr/bin/env python
# -*- coding: utf-8 -*-
# -*- Python -*-

"""
 @file DroneIO.py
 @brief Flight instruction. Get and output the sensor information.
 @date $Date$

 @author 株式会社 東日本計算センター

 プロジェクト方針決定後に記載

"""
import sys
import time
import socket
import math

# Import RTM module
import RTC
import OpenRTM_aist

# drone distance calc
import drone_distance_cal

# Setup logger
from logging import getLogger, StreamHandler
from logging import DEBUG as LOG_LV

# Import DroneKit
from dronekit import connect, VehicleMode, LocationGlobalRelative, Command
from pymavlink import mavutil

sys.path.append(".")

logger = getLogger(__name__)
handler = StreamHandler()
logger.setLevel(LOG_LV)
handler.setLevel(LOG_LV)
logger.addHandler(handler)

# Constant declaration
HOME_LOC_GET_RETRY_CNT = 3  # The max num of downloads of the "home_location".
ARMED_RETRY_CNT = 2  # the max wait time of reflect "armed"
COMM_ERR_TIMER = 30  # communication error(w/ the drone controller)
VEHICLE_DEF_AIR_SPD = 3  # vehicle default speed(unit:m/s)
TAKEOFF_ALT = 10.0 # Takeoff altitude

# drone status declaration
STS_DISARM = 0
STS_ARMED = 1
STS_TAKEOFF = 2
STS_RTL = 3
STS_IN_MOVE = 4
STS_ALT_CHG = 5
STS_HOVERING = 6

# Import Service implementation class
# <rtc-template block="service_impl">

# </rtc-template>

# Import Service stub modules
# <rtc-template block="consumer_import">
# </rtc-template>


# This module's spesification
# <rtc-template block="module_spec">
droneio_spec = ["implementation_id", "DroneIO",
                "type_name", "DroneIO",
                "description",
                "Flight instruction. Get and output the sensor information.",
                "version", "0.1",
                "vendor", "EAC",
                "category", "Drone",
                "activity_type", "STATIC",
                "max_instance", "1",
                "language", "Python",
                "lang_type", "SCRIPT",
                "conf.default.target_ip", "127.0.0.1:14550",
                "conf.__widget__.target_ip", "text",
                ""]
# </rtc-template>

# #
# @class DroneIO
# @brief Flight instruction. Get and output the sensor information.
#
# 別紙(機能仕様書_DroneIO_RTC.docx)参照
#
# 別紙(機能仕様書_DroneIO_RTC.docx)参照
#
# 別紙(機能仕様書_DroneIO_RTC.docx)参照
#
#


class DroneIO(OpenRTM_aist.DataFlowComponentBase):

    # #
    # @brief constructor
    # @param manager Maneger Object
    #
    def __init__(self, manager):
        OpenRTM_aist.DataFlowComponentBase.__init__(self, manager)

        self._d_inTargetPos = RTC.GPSData(
            RTC.Time(0, 0),
            RTC.GPSTime(0, 0),
            0.0, 0.0, 0.0,
            0.0, 0.0,
            0.0, 0.0,
            0.0,
            0,
            RTC.GPS_FIX_NORMAL)
        """
        Drone flight target position
         - Semantics: - Latitude
                      - Longitude
                      - Altitude
                      - air speed(m/s)
        """
        self._inTargetPosIn = OpenRTM_aist.InPort(
            "inTargetPos", self._d_inTargetPos)
        self._d_outGPSDataHome = RTC.GPSData(
            RTC.Time(0, 0),
            RTC.GPSTime(0, 0),
            0.0, 0.0, 0.0,
            0.0, 0.0,
            0.0, 0.0,
            0.0,
            0,
            RTC.GPS_FIX_NORMAL)
        """
        GPS data for "home_location"
         - Semantics: - Latitude
                      - Longitude
                      - Altitude(is relative to mean sea-level)
         - Unit: deg
        """
        self._outGPSDataHomeOut = OpenRTM_aist.OutPort(
            "outGPSDataHome", self._d_outGPSDataHome)
        self._d_outGPSDataDrone = RTC.GPSData(
            RTC.Time(0, 0),
            RTC.GPSTime(0, 0),
            0.0, 0.0, 0.0,
            0.0, 0.0,
            0.0, 0.0,
            0.0,
            0,
            RTC.GPS_FIX_NORMAL)
        """
        GPS data for "Drone"
         - Semantics: - Latitude
                      - Longitude
                      - Altitude(is relative to the "home_location")
         - Unit: deg
        """
        self._outGPSDataDroneOut = OpenRTM_aist.OutPort(
            "outGPSDataDrone", self._d_outGPSDataDrone)
        self._d_outBattery = RTC.TimedFloat(RTC.Time(0, 0), 0)
        """
        Battery voltage
         - Unit: V
        """
        self._outBatteryOut = OpenRTM_aist.OutPort(
            "outBattery", self._d_outBattery)
        self._d_outFlightTime = RTC.Time(RTC.Time(0, 0), 0)
        """
        Drone flight time
         - Unit: sec
        """
        self._outFlightTimeOut = OpenRTM_aist.OutPort(
            "outFlightTime", self._d_outFlightTime)
        self._d_outGyro = RTC.TimedFloatSeq(RTC.Time(0, 0), [])
        """
        Gyro sensor
         - Semantics: - Pitch Axis
                      - Roll Axis
                      - Yaw Axis
        """
        self._outGyroOut = OpenRTM_aist.OutPort("outGyro", self._d_outGyro)
        self._d_outAccelero = RTC.TimedFloatSeq(RTC.Time(0, 0), [])
        """
        Accelerometer
         - Semantics: - x
                      - y
                      - z
         - Unit: m/sec
        """
        self._outAcceleroOut = OpenRTM_aist.OutPort(
            "outAccelero", self._d_outAccelero)
        self._d_outCompass = RTC.TimedFloat(RTC.Time(0, 0), 0)
        """
        Magnetic Compass
         - Semantics: 0-360.
                      (North is 0, East is 90, South is 180, West is 270)
         - Unit: deg
        """
        self._outCompassOut = OpenRTM_aist.OutPort(
            "outCompass", self._d_outCompass)
        self._d_outTemp = RTC.TimedFloat(RTC.Time(0, 0), 0)
        """
        Temperature
         - Unit: degree Celsius
        """
        self._outTempOut = OpenRTM_aist.OutPort("outTemp", self._d_outTemp)
        self._d_outPres = RTC.TimedFloat(RTC.Time(0, 0), 0)
        """
        Pressure
         - Unit: Pa
        """
        self._outPresOut = OpenRTM_aist.OutPort("outPres", self._d_outPres)

        # initialize of configuration-data.
        # <rtc-template block="init_conf_param">
        """
        To connecting to the target vehicle.
        (Set up within the DroneKit script.)
         - Name: target_ip target_ip
         - DefaultValue: 127.0.0.1:14550
        """
        self._target_ip = ['127.0.0.1:14550']

        # </rtc-template>

        self._vehicle = None  # vehicle object
        self._flight_time_start = 0  # Flight Start Time
        self._drone_sts = STS_DISARM
        self._prev_armed = False  # previous armed status
        self._comm_err_timer = 0  # communication error check timer
        self._takeoff_alt = 0
        self._alt_chg_req_alt = 0
        self._move_target_lat = 0
        self._move_target_lon = 0

    # #
    # RTC Initialize
    #
    # The initialize action (on CREATED->ALIVE transition)
    # formaer rtc_init_entry()
    #
    # @return RTC::ReturnCode_t
    #
    #
    def onInitialize(self):
        # Bind variables and configuration variable
        self.bindParameter("target_ip", self._target_ip, "127.0.0.1:14550")

        # Set InPort buffers
        self.addInPort("inTargetPos", self._inTargetPosIn)

        # Set OutPort buffers
        self.addOutPort("outGPSDataHome", self._outGPSDataHomeOut)
        self.addOutPort("outGPSDataDrone", self._outGPSDataDroneOut)
        self.addOutPort("outBattery", self._outBatteryOut)
        self.addOutPort("outFlightTime", self._outFlightTimeOut)
        self.addOutPort("outGyro", self._outGyroOut)
        self.addOutPort("outAccelero", self._outAcceleroOut)
        self.addOutPort("outCompass", self._outCompassOut)
        self.addOutPort("outTemp", self._outTempOut)
        self.addOutPort("outPres", self._outPresOut)

        # Set service provider to Ports

        # Set service consumers to Ports

        # Set CORBA Service Ports

        return RTC.RTC_OK

        # #
        # Close vehicle object
        #
        # The finalize action (on ALIVE->END transition)
        # formaer rtc_exiting_entry()
        #
        # @return RTC::ReturnCode_t

        #
    def onFinalize(self):

        self._vehicle.close()
        logger.info("vehicle close")

        return RTC.RTC_OK

        # #
        # - Connecting to the vehicle(default target adress: 127.0.0.1:14550).
    #     - Download the "home_location"
        #
        # The startup action when ExecutionContext startup
        # former rtc_starting_entry()
        #
        # @param ec_id target ExecutionContext Id
        #
        # @return RTC::ReturnCode_t
        #
        #
    def onStartup(self, ec_id):

        if self.connecting_vehicle() is True:
            if self.get_home_location() is True:
                return RTC.RTC_OK
            else:
                return RTC.RTC_ERROR
        else:
            return RTC.RTC_ERROR

    #     ##
    #     #
    #     # The shutdown action when ExecutionContext stop
    #     # former rtc_stopping_entry()
    #     #
    #     # @param ec_id target ExecutionContext Id
    #     #
    #     # @return RTC::ReturnCode_t
    #     #
    #     #
    # def onShutdown(self, ec_id):
    #
    #     return RTC.RTC_OK

        # #
        # - Start the counting of the flight timer.
    #     - Battery parameter setting
    #     - Vehicle mode is set to GUIDED
    #     - Vehicle.armed status is set to True(arm)
        #
        # The activated action (Active state entry action)
        # former rtc_active_entry()
        #
        # @param ec_id target ExecutionContext Id
        #
        # @return RTC::ReturnCode_t
        #
        # @pre "home_location" exists
        #
    def onActivated(self, ec_id):

        if self._drone_sts != STS_RTL:
            # home_location check
            if self._vehicle.home_location is not None:
                # Change the vehicle mode
                self._vehicle.mode = VehicleMode("GUIDED")

                # Change the arm status
                if self.switching_to_armed() is True:
                    self._drone_sts = STS_ARMED
                    # Flight time count to start
                    self._flight_time_start = time.time()
                    # Battery parameters tuning
                    # Controls enabling monitoring of the battery's voltage
                    # only
                    self._vehicle.parameters['BATT_MONITOR'] = 3
                    # Voltage Sensing Pin(Pixhawk)
                    self._vehicle.parameters['BATT_VOLT_PIN'] = 2
                    return RTC.RTC_OK
                else:
                    return RTC.RTC_ERROR
            else:
                logger.error("[ERROR]home_location is not exist. "
                             "Please \"Exit\" this RTC.")
                return RTC.RTC_ERROR
        else:
            return RTC.RTC_OK

        # #
        # - Vehicle.armed status is set to False(disarm)
        #
        # The deactivated action (Active state exit action)
        # former rtc_active_exit()
        #
        # @param ec_id target ExecutionContext Id
        #
        # @return RTC::ReturnCode_t
        #
        #
    def onDeactivated(self, ec_id):

        if self._drone_sts != STS_RTL:
            if (self._drone_sts != STS_DISARM and
                    self._drone_sts != STS_ARMED):
                self._vehicle.mode = VehicleMode("RTL")
                self._drone_sts = STS_RTL
                logger.info("RTC deactivated. "
                            "Start the \"Return To Launch\".")
            else:
                self._vehicle.armed = False
                self._drone_sts = STS_DISARM
                logger.info("RTC deactivated. "
                            "Switching to disarm.")
        else:
            logger.info("RTC deactivated. "
                        "Will be continued \"Return To Launch\".")

        return RTC.RTC_OK

        # #
        # - Get and output of the sensor info through the DroneKit-Python API.
    #     - Flight instruction through the DronKit-Python API.
        #
        # The execution action that is invoked periodically
        # former rtc_active_do()
        #
        # @param ec_id target ExecutionContext Id
        #
        # @return RTC::ReturnCode_t
        #
        #
    def onExecute(self, ec_id):

        rtn = True

        # Armed Check
        if self._vehicle.armed is True:
            # Read the target position
            if self._inTargetPosIn.isNew():
                self._comm_err_timer = 0
                self.flight_ctrl_core()
            else:
                # Communication error check
                self._comm_err_timer += 1
                if self._comm_err_timer == COMM_ERR_TIMER:
                    logger.error("[ERROR:Timeout]Communication error "
                                 "w/ the drone controller.")
                    self._comm_err_timer = 0
                    rtn = False

            # judge the target position arrive
            self.target_pos_arrive_judge()

            # Read & output the sensor info
            self.writeSensorInfo()
        else:
            # judgement switching from armed to disarm
            if self.arm_chg_judge() is not True:
                # always disarm
                rtn = False

        # update armed status
        self._prev_armed = self._vehicle.armed

        if rtn is True:
            return RTC.RTC_OK
        else:
            logger.debug("onExecute ERROR")
            return RTC.RTC_ERROR

    #     ##
    #     #
    #     # The aborting action when main logic error occurred.
    #     # former rtc_aborting_entry()
    #     #
    #     # @param ec_id target ExecutionContext Id
    #     #
    #     # @return RTC::ReturnCode_t
    #     #
    #     #
    # def onAborting(self, ec_id):
    #
    #     return RTC.RTC_OK

        # #
        # - Return To Launch
        #
        # The error action in ERROR state
        # former rtc_error_do()
        #
        # @param ec_id target ExecutionContext Id
        #
        # @return RTC::ReturnCode_t
        #
        #
    def onError(self, ec_id):

        if (self._drone_sts != STS_DISARM and
                self._drone_sts != STS_ARMED and
                self._drone_sts != STS_RTL):
            self._vehicle.mode = VehicleMode("RTL")
            self._drone_sts = STS_RTL
            logger.error("[ERROR]RTC onError. "
                         "Start the \"Return To Launch\".")

        return RTC.RTC_OK

    #     ##
    #     #
    #     # The reset action that is invoked resetting
    #     # This is same but different the former rtc_init_entry()
    #     #
    #     # @param ec_id target ExecutionContext Id
    #     #
    #     # @return RTC::ReturnCode_t
    #     #
    #     #
    # def onReset(self, ec_id):
    #
    #     return RTC.RTC_OK

    #     ##
    #     #
    #     # The state update action that is invoked after onExecute() action
    #     # no corresponding operation exists in OpenRTm-aist-0.2.0
    #     #
    #     # @param ec_id target ExecutionContext Id
    #     #
    #     # @return RTC::ReturnCode_t
    #     #

    #     #
    # def onStateUpdate(self, ec_id):
    #
    #     return RTC.RTC_OK

    #     ##
    #     #
    #     # The action that is invoked when execution context's rate is changed
    #     # no corresponding operation exists in OpenRTm-aist-0.2.0
    #     #
    #     # @param ec_id target ExecutionContext Id
    #     #
    #     # @return RTC::ReturnCode_t
    #     #
    #     #
    # def onRateChanged(self, ec_id):
    #
    #     return RTC.RTC_OK

    def connecting_vehicle(self):

        # Connecting to the vehicle
        rtn = True

        try:
            logger.info("\nConnecting to vehicle on: %s",
                        self._target_ip[0])
            self._vehicle = connect(self._target_ip[0], wait_ready=True)
        except socket.error:
            logger.error("[ERROR]Socket error. "
                         "Please \"Exit\" this RTC.")
            rtn = False
        except dronekit.APIException:
            logger.error("[ERROR]Connection Timeout. "
                         "Please \"Exit\" this RTC.")
            rtn = False

        return rtn

    def get_home_location(self):
        # Get vehicle Home Location - will be "None" until 1st set by
        # autopilot.(Max 3sec)
        rtn = True

        try:
            retry_cnt = 0
            while retry_cnt < HOME_LOC_GET_RETRY_CNT:
                cmds = self._vehicle.commands
                cmds.download()
                cmds.wait_ready()
                if not self._vehicle.home_location:
                    retry_cnt += 1
                    logger.info("Waiting for home location...")
                    time.sleep(1)
                else:
                    logger.info("%s", self._vehicle.home_location)
                    break

            if retry_cnt == HOME_LOC_GET_RETRY_CNT:
                logger.error("[ERROR]home_location timeout. "
                             "Please \"Exit\" this RTC.")
                rtn = False

        except:
            logger.error("[ERROR]Please \"Exit\" this RTC.")
            rtn = False

        return rtn

    def switching_to_armed(self):

        rtn = True

        self._vehicle.armed = True
        retrycnt = 0
        while retrycnt < ARMED_RETRY_CNT:
            if self._vehicle.armed is not True:
                retrycnt += 1
                logger.info("Waiting for arming...")
                time.sleep(1)
            else:
                logger.info("Switching to armed success.")
                break

        if retrycnt == ARMED_RETRY_CNT:
            logger.error("[ERROR]Timeout. Switching to armed failed. "
                         "Please \"Exit\" this RTC.")
            rtn = False

        return rtn

    def flight_ctrl_core(self):

        self._d_inTargetPos = self._inTargetPosIn.read()

        # Setting the air speed
        if self._d_inTargetPos.horizontalSpeed != 0:
            self._vehicle.airspeed = self._d_inTargetPos.horizontalSpeed
        else:
            self._vehicle.airspeed = VEHICLE_DEF_AIR_SPD

        if self._drone_sts == STS_ARMED:
            if self._d_inTargetPos.altitude == TAKEOFF_ALT:
                # take off
                self._takeoff_alt = self._d_inTargetPos.altitude
                self._vehicle.simple_takeoff(self._takeoff_alt)
                self._drone_sts = STS_TAKEOFF
                logger.info("Start the takeoff")
            else:
                logger.error("[ERROR]Wrong takeoff parameter: %f",
                             self._d_inTargetPos.altitude)
        elif (self._drone_sts == STS_IN_MOVE or
              self._drone_sts == STS_ALT_CHG or
              self._drone_sts == STS_HOVERING):
            # The state is in move(contain altitude change) or hovering.
            if self.home_target_pos_same_judge() is True:
                # RTL if the target pos is same the home pos
                self._vehicle.mode = VehicleMode("RTL")
                self._drone_sts = STS_RTL
                logger.info("Start the \"Return To Launch\".")
            else:
                self.move_to_target_pos()

        else:
            # None, if the state is RTL or take off
            pass

        logger.debug("drone_sts: %d", self._drone_sts)
        logger.debug("vehivle.mode: %s", self._vehicle.mode)

    def home_target_pos_same_judge(self):

        rtn = False

        if self._d_inTargetPos.latitude == self._d_outGPSDataHome.latitude:
            if self._d_inTargetPos.longitude == self._d_outGPSDataHome.longitude:
                if self._d_inTargetPos.altitude == self._d_outGPSDataHome.altitude:
                    rtn = True

        return rtn

    def move_to_target_pos(self):
        logger.debug("Drone lat:%.10f lon:%.10f alt:%.10f",
                     self._d_outGPSDataDrone.latitude,
                     self._d_outGPSDataDrone.longitude,
                     self._d_outGPSDataDrone.altitude)

        logger.debug("Target lat:%.10f lon:%.10f alt:%.10f",
                     self._d_inTargetPos.latitude,
                     self._d_inTargetPos.longitude,
                     self._d_inTargetPos.altitude)

        dist_cal = drone_distance_cal.DroneMathmetics()
        dist = dist_cal.distance_math(self._d_outGPSDataDrone.longitude,
                                      self._d_inTargetPos.longitude,
                                      self._d_outGPSDataDrone.latitude,
                                      self._d_inTargetPos.latitude,
                                      0,
                                      0)

        logger.debug("Target dist:%f[m]", dist)

        # "2" means mininum move distance
        # See "機能仕様書_DroneController.docx" for details
        if dist <= 2:
            # altitude change, if change value is allowable range
            self._alt_chg_req_alt = self._d_inTargetPos.altitude
            self._drone_sts = STS_ALT_CHG
            logger.debug("altitude change: %f",
                         self._d_inTargetPos.altitude)
        else:
            self._move_target_lat = self._d_inTargetPos.latitude
            self._move_target_lon = self._d_inTargetPos.longitude
            self._drone_sts = STS_IN_MOVE
            logger.debug("Move to :%f %f %f",
                         self._d_inTargetPos.latitude,
                         self._d_inTargetPos.longitude,
                         self._d_inTargetPos.altitude)

        # Move to the target point
        point = LocationGlobalRelative(
            self._d_inTargetPos.latitude,
            self._d_inTargetPos.longitude,
            self._d_inTargetPos.altitude)
        self._vehicle.simple_goto(point)

    def target_pos_arrive_judge(self):

        if self._drone_sts == STS_TAKEOFF:
            logger.debug("Current Alt:%f", self._d_outGPSDataDrone.altitude)
            if self._takeoff_alt * 0.95 <= self._d_outGPSDataDrone.altitude:
                logger.info("takeoff complete")
                self._drone_sts = STS_HOVERING

        elif self._drone_sts == STS_IN_MOVE:
            dist_cal = drone_distance_cal.DroneMathmetics()
            dist = dist_cal.distance_math(self._d_outGPSDataDrone.longitude,
                                          self._move_target_lon,
                                          self._d_outGPSDataDrone.latitude,
                                          self._move_target_lat,
                                          0,
                                          0)

            logger.debug("Drone<->Target dist:%f", dist)

            if dist <= 1:
                logger.info("Arrived at the target position")
                self._drone_sts = STS_HOVERING

        elif self._drone_sts == STS_ALT_CHG:
            tgt_pos_min = self._alt_chg_req_alt * 0.95
            tgt_pos_max = self._alt_chg_req_alt * 1.05
            logger.debug("Current Alt:%f", self._d_outGPSDataDrone.altitude)
            if tgt_pos_min <= self._d_outGPSDataDrone.altitude <= tgt_pos_max:
                logger.info("Arrived at the target alt")
                self._drone_sts = STS_HOVERING

        elif self._drone_sts == STS_RTL:
            logger.debug("Current Alt:%f", self._d_outGPSDataDrone.altitude)

        else:
            # The state is Armed
            pass

    def arm_chg_judge(self):

        rtn = False

        if self._prev_armed is True:
            self._drone_sts = STS_DISARM
            rtn = True

        return rtn

    def writeSensorInfo(self):

        self.writeGPSdataHome()
        self.writeGPSdataDrone()
        self.writeBattery()
        self.writeFlightTime()
        self.writeGyro()
        self.writeAccelerometer()
        self.writeMagCompass()
        self.writeTemperature()
        self.writePressure()

    def writeGPSdataHome(self):
        # GPS(Home)
        if self._vehicle.home_location is not None:
            tmp_home_location = self._vehicle.home_location
            self._d_outGPSDataHome.tm.sec = 0
            self._d_outGPSDataHome.tm.nsec = 0
            self._d_outGPSDataHome.timeFromGPS.sec = 0
            self._d_outGPSDataHome.timeFromGPS.msec = 0
            self._d_outGPSDataHome.latitude = tmp_home_location.lat
            self._d_outGPSDataHome.longitude = tmp_home_location.lon
            self._d_outGPSDataHome.altitude = tmp_home_location.alt
            self._d_outGPSDataHome.horizontalError = 0.0
            self._d_outGPSDataHome.verticalError = 0.0
            self._d_outGPSDataHome.heading = 0.0
            self._d_outGPSDataHome.horizontalSpeed = 0.0
            self._d_outGPSDataHome.verticalSpeed = 0.0
            self._d_outGPSDataHome.numSatellites = 0
            self._d_outGPSDataHome.fixType = RTC.GPS_FIX_NORMAL
            self._outGPSDataHomeOut.write()

    def writeGPSdataDrone(self):
        # GPS(Drone)
        self._d_outGPSDataDrone.tm.sec = 0
        self._d_outGPSDataDrone.tm.nsec = 0
        self._d_outGPSDataDrone.timeFromGPS.sec = 0
        self._d_outGPSDataDrone.timeFromGPS.msec = 0

        if self._vehicle.location.global_relative_frame.lat is not None:
            self._d_outGPSDataDrone.latitude = float(
                self._vehicle.location.global_relative_frame.lat)
        else:
            self._d_outGPSDataDrone.latitude = 0.0

        if self._vehicle.location.global_relative_frame.lon is not None:
            self._d_outGPSDataDrone.longitude = float(
                self._vehicle.location.global_relative_frame.lon)
        else:
            self._d_outGPSDataDrone.longitude = 0

        if self._vehicle.location.global_relative_frame.alt is not None:
            self._d_outGPSDataDrone.altitude = float(
                self._vehicle.location.global_relative_frame.alt)
        else:
            self._d_outGPSDataDrone.altitude = 0.0

        self._d_outGPSDataDrone.horizontalError = 0.0
        self._d_outGPSDataDrone.verticalError = 0.0
        self._d_outGPSDataDrone.heading = 0.0
        self._d_outGPSDataDrone.horizontalSpeed = 0.0
        self._d_outGPSDataDrone.verticalSpeed = 0.0
        self._d_outGPSDataDrone.numSatellites = 0
        self._d_outGPSDataDrone.fixType = RTC.GPS_FIX_NORMAL
        self._outGPSDataDroneOut.write()
        # print "%s" % self._vehicle.location.global_relative_frame

    def writeBattery(self):
        # Battery(V)
        if self._vehicle.battery.voltage is not None:
            self._d_outBattery.data = self._vehicle.battery.voltage
        else:
            self._d_outBattery.data = 0.0
        self._outBatteryOut.write()
        logger.debug("%s", self._vehicle.battery)
        #print "%s" % self._vehicle.battery

    def writeFlightTime(self):
        # Flight Time
        if self._flight_time_start != 0:
            current_flight_time = time.time() - self._flight_time_start
            self._d_outFlightTime.sec = int(current_flight_time)
        else:
            self._d_outFlightTime.sec = 0
        self._outFlightTimeOut.write()
        min = self._d_outFlightTime.sec / 60
        sec = self._d_outFlightTime.sec % 60
        logger.debug("Time: %02d:%02d", min, sec)

    def writeGyro(self):
        # Gyro
        if self._vehicle.attitude.pitch is not None:
            tmp_pitch = self._vehicle.attitude.pitch
        else:
            tmp_pitch = 0.0

        if self._vehicle.attitude.roll is not None:
            tmp_roll = self._vehicle.attitude.roll
        else:
            tmp_roll = 0.0

        if self._vehicle.attitude.yaw is not None:
            tmp_yaw = self._vehicle.attitude.yaw
        else:
            tmp_yaw = 0.0

        self._d_outGyro.data = [tmp_pitch, tmp_roll, tmp_yaw]
        self._outGyroOut.write()
        logger.debug("%s", self._vehicle.attitude)
        # print "pitch:%s, roll:%s, yaw:%s" % (tmp_pitch, tmp_roll, tmp_yaw)

    def writeAccelerometer(self):
        # Accelerometer
        tmp_acc = self._vehicle.velocity
        if tmp_acc[0] is None:
            tmp_acc[0] = 0.0

        if tmp_acc[1] is None:
            tmp_acc[1] = 0.0

        if tmp_acc[2] is None:
            tmp_acc[2] = 0.0

        self._d_outAccelero.data = tmp_acc
        self._outAcceleroOut.write()
        logger.debug("Accelerometer: %s", self._vehicle.velocity)
        # print "vx:%s, vy:%s, vz:%s" % (tmp_acc[0], tmp_acc[1], tmp_acc[2])

    def writeMagCompass(self):
        # Magnetic Compass
        if self._vehicle.heading is not None:
            self._d_outCompass.data = self._vehicle.heading
        else:
            self._d_outCompass.data = 0.0

        self._outCompassOut.write()
        logger.debug("Compass:%d", self._d_outCompass.data)
        # print "mag:%s" % self._d_outCompass.data

    def writeTemperature(self):
        # Temperature
        if self._vehicle.parameters['GND_TEMP'] is not None:
            self._d_outTemp.data = self._vehicle.parameters['GND_TEMP']
        else:
            self._d_outTemp.data = 0.0

        self._outTempOut.write()
        logger.debug("Temperature:%s[C]", self._d_outTemp.data)
        # print "Temp:%s" % self._d_outTemp.data

    def writePressure(self):
        # Pressure(Pa)
        if self._vehicle.parameters['GND_ABS_PRESS'] is not None:
            self._d_outPres.data = self._vehicle.parameters['GND_ABS_PRESS']
        else:
            self._d_outPres.data = 0.0

        self._outPresOut.write()
        logger.debug("Pressure:%s[Pa]", self._d_outPres.data)
        # print "Pres:%s" % self._d_outPres.data


def DroneIOInit(manager):
    profile = OpenRTM_aist.Properties(defaults_str=droneio_spec)
    manager.registerFactory(profile,
                            DroneIO,
                            OpenRTM_aist.Delete)


def MyModuleInit(manager):
    DroneIOInit(manager)

    # Create a component
    comp = manager.createComponent("DroneIO")


def main():
    mgr = OpenRTM_aist.Manager.init(sys.argv)
    mgr.setModuleInitProc(MyModuleInit)
    mgr.activateManager()
    mgr.runManager()

if __name__ == "__main__":
    main()
